#ifndef gws_GENERICWORKSPACE_HPP_INCLUDED
#define gws_GENERICWORKSPACE_HPP_INCLUDED 1

#include <map>

#include <qworkspace.h>
#include <qframe.h>
#include <qwidget.h>
#include <qsize.h>
#include <qpoint.h>

namespace gws {

        /**
           GenericWorkspaceWindow should not be used by client
           code. It is in this header so moc will pick it up. :/
        */
        class GenericWorkspaceWindow : public QFrame
        {
                Q_OBJECT;
        public:
                GenericWorkspaceWindow( QWidget * child, QWidget * parent );
                virtual ~GenericWorkspaceWindow();
                QWidget * child() { return this->m_child; }

                virtual QSize sizeHint();
                virtual void showMaximized();

        protected:
                virtual void closeEvent( QCloseEvent *e );

        private:
                void init();
                QWidget * m_child;
        };

        /**
           A slightly customized QWorkspace.
        */
        class GenericWorkspace : public QWorkspace
        {
                Q_OBJECT;
        public:


                GenericWorkspace( QWidget * parent = 0 );
                virtual ~GenericWorkspace();

//                 typedef std::map<QWidget *,GenericWorkspaceWindow *> window_map;
//                 window_map & windows();

        public slots:
                /**
                   Wraps the given widget in a GenericWorkspaceWindow
                   and adds it to the workspace.

                   Please use this function to add child widgets, instead
                   of reparent(), because this ensures that the objects
                   are deleted when their MDI window is closed.
                 */
                void addWidget( QWidget * );

        private slots:
                void childDying( QObject * );

        private:
                typedef std::pair<QWidget *,GenericWorkspaceWindow *> pair_type;
                /**
                   This map type maps client-added QWidgets to
                   the actual parent container widget which is used
                   in the workspace.

                   The key type is QObject for reasons involving child
                   destruction notification - that's the highest level
                   of info we get. When a QWidget is deleted, we
                   can't be notified until after it's QWidget part is
                   destroyed, but we do get it's QObject part.

                   So... this map maps (QObject *) to (QWidget *).
                   The QObjects are guaranteed to be QWidgets except
                   during (or after) destruction of those objects.

                   This is an internal detail, in any case.

                   TODO: need a public way of getting at the list
                   of child widgets. Qt will only directly let us
                   at the containing "window" widgets.
                */
                typedef std::map<QObject *,pair_type> qobject_map;
                void init();
                qobject_map m_qobjects;
                /**
                   We have this map so that we can provide a set of
                   algos for doing things like getting lists of
                   client-supplied widgets or the MDI-window container
                   widgets... things like that.
                */
                qobject_map & qobjects();
        };


} // namespace gws


#endif // gws_GENERICWORKSPACE_HPP_INCLUDED
