#ifdef NDEBUG
#  undef NDEBUG // force assert() to work
#endif
#include <cassert>
#include <memory> // auto_ptr
#include <list>

#include <stdlib.h> // getenv()

#include <qt.h>


// #define s11n_NO_DEFAULT_POD_REGISTRATIONS 1
// #include <s11n.net/s11n/s11nlite.hpp>
#include <s11n.net/s11n/s11n_debuggering_macros.hpp> // CERR
#include <s11n.net/acme/argv_parser.hpp>
#include <s11n.net/launch/launch.hpp>
#include <s11n.net/cl/cllite.hpp>
#include <s11n.net/cl/dll_util.hpp>
#include <s11n.net/s11n/s11nlite.hpp>

#include <qstring.h>
#include <qmessagebox.h>

#include "GenericWorkspace.h"

#include "genericmainwindow.h"


#include "ws.hpp"
#include "qt_serialize.hpp"


namespace gws {

        GenericWorkspace * m_workspace = 0;
        GenericWorkspace * theWorkspace()
        {
                return m_workspace;
        }

        bool addWidget( QWidget * t )
        {
//                 CERR << "gwd::addWidget("<<std::hex<<t<<")\n";
                if( ! gws::theWorkspace() )
                {
                        return false;
                }
                gws::theWorkspace()->addWidget( t );
                return true;
        }

        void messageBox( const QString & caption, const QString & msg )
        {
                QMessageBox::information( 0, caption, msg, 0 );
        }


        QWidget * createWidget( const std::string & classname )
        {
                return cllite::classload<QWidget>( classname );
        }
} // namespace gws





// #include <s11n.net/s11n/pods_streamable.hpp>
// #include <s11n.net/s11n/list.hpp>
// void testClipboard()
// {
//         typedef std::list<int> LT;
//         LT il;
//         for( int i = 0; i < 5; i++ ) il.push_back(i);
//         bool worked = s11n::qt::serializableToClipboard<s11nlite::node_type,LT>( il );
//         CERR << "to clipboard? == " << worked << "\n";
//         LT * dli = s11n::qt::serializableFromClipboard<s11nlite::node_type,LT>();
//         CERR << "from clipboard? = " << std::hex << dli << "\n";
//         if( dli )
//         {
//                 s11nlite::save( *dli, std::cerr );
//         }
//         delete dli;
// }

#define CFGKEY_SIZE "GenericWorkspace_size"


int main( int argc, char ** argv )
{
        enum {
        ErrorDLL = 1,
        ErrorWidgetLoad
        };


        std::list<cllite::dll::dlentry> dllist;
        size_t dlloads = cllite::dll::parse_dlentry_flags( argc - 1, argv + 1, dllist );
//         acme::argv_parser & args =
                acme::argv_parser::args( argc, argv );
        QApplication app( argc, argv );

        GenericMainWindow * win = new GenericMainWindow( 0, "Generic Workspace" );
        app.setMainWidget( win );

        typedef gws::GenericWorkspace WS;
        WS * ws = new WS( win );
        gws::m_workspace = ws;

        win->setCentralWidget( ws );


        QSize wsz = s11nlite::config().get( CFGKEY_SIZE, win->sizeHint() );
        win->resize( wsz );


        if( 0 != dlloads )
        {
                try
                {
                        cllite::dll::dlload( dllist );
                }
                catch( const cllite::cl_exception & ex )
                {
                        CERR << ex.what() << "\n";
                        return ErrorDLL;
                }
        }

        typedef std::list<std::string> SList;

        SList widgets;
        std::string arg;
        for( int i = 0; i < argc-1; i++ )
        {
                arg = argv[i];
                if( arg != "-w" ) continue;
                widgets.push_back( argv[i+1] );
                ++i; // skip widget name
        }



        win->show();

        QWidget * ch = 0;
        SList::const_iterator wit = widgets.begin(), wet = widgets.end();
        for( ; wet != wit; ++wit )
        {
                ch = gws::createWidget( (*wit) );
                if( ! ch )
                {
                        gws::messageBox( "Classload failed!", QString("Loading widget '")+(*wit)+"' failed. :(");
                        return ErrorWidgetLoad;
                }
                ws->addWidget( ch );
        }
//         testClipboard();
        int ret = app.exec();
        s11nlite::config().set( CFGKEY_SIZE, win->size() );
        delete ws;
        return ret;
}

